/* ---- mixer-2.1.mono.orc - written by Istvan Varga, Jun 2002 ---- */

sr	=  48000
ksmps	=  64
nchnls	=  1

gidel	=  0.25			; output delay
gimchn	=  10			; master channel

; -----------------------------------------------------------------------------

; assign all MIDI channels to instr 3, except the master channel
; which is assigned to a dummy instrument

ichn	=  1
l1:
	massign ichn, (abs(ichn - gimchn) < 0.5 ? 2 : 3)
ichn	=  ichn + 1
	if (ichn < 16.5) igoto l1

; disable all program changes
	pgmassign 0, 0

; 0 - 2047:    controller data (16 * 128 controllers)
; 4096 - 4351: channel parameters (16 * 16 values)
; 16384 - :    delay lines for controllers

	zakinit 1, 1000000

gizkd	=  int(gidel * kr + 0.5)	; controller delay in samples
gizkds	=  gizkd + 1			; delay line length

gaxL	init 0		; initialize global a-rate accumulators

; scale for random noise to avoid denormalized numbers
girscl	=  0.000001 * 0.000001 * 0.000001 * 0.000001

; read controller value from zak space

#define ZCTRL(kr'chn'ctrl) #

iz_$chn._$ctrl	=  int($chn * 128 + $ctrl - 127.5)
$kr		zkr iz_$chn._$ctrl

#

; tables for converting MIDI controllers

itmp	ftgen 1, 0, 129, -5, 0.03125, 129, 8		; volume 2
itmp	ftgen 2, 0, 129, -5, 16.352, 129, 26579.5	; EQ frequency
itmp	ftgen 3, 0, 129, -5, 0.0625, 129, 16		; EQ level
itmp	ftgen 4, 0, 129, -5, 0.04419, 129, 11.3137	; EQ Q

/* -------- instr 1: read and process controller data -------- */

	instr 1

ichn	=  p4				; channel number

; store channel parameters in zak space

	ziw  p5, int(4080.5 + ichn * 16)	; amplitude scale
	ziw  p6, int(4081.5 + ichn * 16)	; skip time
	ziw  p7, int(4082.5 + ichn * 16)	; delay offset
	ziw  p8, int(4083.5 + ichn * 16)	; extra time for reverb etc.
	ziw p12, int(4084.5 + ichn * 16)	; EQ 1 mode
	ziw p16, int(4085.5 + ichn * 16)	; EQ 2 mode

kzrpos	init 0			; zak read position
kzwpos	init gizkd		; zak write position

itim1	=  0.04			; controller interpolation time
ktrig1	phasor 1 / itim1
ktrig1	trigger 1 - ktrig1, 0.5, 0	; trigger signal for interpolation

#define CTRL(num'pnum'ctrl) #

; zak offset for controller delay line
izofs_$num	=  int(((ichn - 1) * 9 + ($num - 1)) * gizkds + 16384.5)
; initialize controller
ictlz_$num	=  int(ichn * 128 + $ctrl - 127.5)	; zak location
		ziw p$pnum, ictlz_$num
		ctrlinit ichn, $ctrl, p$pnum
kctl_$num	init p$pnum

; get controller data

ktmp		ctrl7 ichn, $ctrl, 0, 127
kctl_$num	=  (ktmp < 0.5 ? kctl_$num : ktmp)

; delay and interpolation
	zkw kctl_$num - p$pnum, izofs_$num + kzwpos
ktmp	zkr izofs_$num + kzrpos
ktmp	tlineto ktmp + p$pnum, itim1, ktrig1

; store delayed and interpolated controller value in zak space
	zkw ktmp, ictlz_$num

#

$CTRL(1'9'7)		; volume
$CTRL(2'10'24)		; volume 2
$CTRL(3'11'25)		; delay
$CTRL(4'13'26)		; EQ 1 frequency
$CTRL(5'14'27)		; EQ 1 level
$CTRL(6'15'28)		; EQ 1 Q
$CTRL(7'17'29)		; EQ 2 frequency
$CTRL(8'18'30)		; EQ 2 level
$CTRL(9'19'31)		; EQ 2 Q

; update zak read and write position
kzrpos	=  (kzrpos < (gizkd - 0.5) ? kzrpos + 1 : 0)
kzwpos	=  (kzwpos < (gizkd - 0.5) ? kzwpos + 1 : 0)

	endin

/* -------- instr 2: dummy instrument -------- */

	instr 2
	endin

/* -------- instr 3: read input files -------- */

	instr 3

ichn	midichn			; channel number
iflnum	notnum			; file number

iamp	zir int(4080.5 + ichn * 16)		; amplitude scale
iskptim	zir int(4081.5 + ichn * 16)		; skip time
idel	zir int(4082.5 + ichn * 16)		; delay offset
ixtim	zir int(4083.5 + ichn * 16)		; extra time
imode1	zir int(4084.5 + ichn * 16)		; EQ 1 mode
imode2	zir int(4085.5 + ichn * 16)		; EQ 2 mode

; extend note duration
ixtim	=  ixtim + gidel + (2 / kr)
	xtratim ixtim
; convert time offset to delay
idel	=  (int((gidel - idel) * sr + 0.5) + 0.03) / sr

; de-click envelope

arel	init 1
ktmp	release
	if (ktmp < 0.5) kgoto l03
arel	linseg 1, ixtim - (0.02 + (2 / kr)), 1, 0.02, 0, 1, 0
l03:

; get input from sound file

a3	soundin int(iflnum + 0.5), iskptim

a1	rnd31 girscl, 0, ichn * 2		; add low level noise
	vincr a3, a1				; to avoid denormals

; read and convert controllers

$ZCTRL(kvol'ichn'7)			; volume 1
$ZCTRL(kvol2'ichn'24)			; volume 2
$ZCTRL(kdel'ichn'25)			; delay

kvol	=  kvol * kvol / 16129
kvol2	tablei kvol2, 1
kdel	=  (kdel - 64) * 0.001

; delay

kdel	limit kdel + idel, (ksmps + 1) / sr, gidel - ((ksmps + 1) / sr)
kdel	=  (int(kdel * sr + 0.5) + 0.25) / sr
atmp	delayr gidel
aL	deltap kdel
	delayw a3

; apply amplitude control and parametric equalizers (if enabled)

kvol	=  iamp * kvol * kvol2

aL	=  aL * kvol

	if (imode1 < -0.5) goto l01
$ZCTRL(keq1f'ichn'26)			; EQ 1 frequency
$ZCTRL(keq1l'ichn'27)			; EQ 1 level
$ZCTRL(keq1q'ichn'28)			; EQ 1 Q
keq1f	tablei keq1f, 2			; convert controller values
keq1l	tablei keq1l, 3
keq1q	tablei keq1q, 4
aL	pareq aL, keq1f, keq1l, keq1q, imode1
l01:
	if (imode2 < -0.5) goto l02
$ZCTRL(keq2f'ichn'29)			; EQ 2 frequency
$ZCTRL(keq2l'ichn'30)			; EQ 2 level
$ZCTRL(keq2q'ichn'31)			; EQ 2 Q
keq2f	tablei keq2f, 2			; convert controller values
keq2l	tablei keq2l, 3
keq2q	tablei keq2q, 4
aL	pareq aL, keq2f, keq2l, keq2q, imode2
l02:

	vincr gaxL, aL * arel		; de-click and send to output

	endin

/* -------- instr 90: output instrument -------- */

	instr 90

ichn	=  gimchn		; channel number

iamp	zir int(4080.5 + ichn * 16)		; amplitude scale
imode1	zir int(4084.5 + ichn * 16)		; EQ 1 mode
imode2	zir int(4085.5 + ichn * 16)		; EQ 2 mode

; de-click envelope

krel	linseg 0, p3 - (0.02 + (2 / kr)), 0, 0.02, 1, 1, 1
arel	interp krel
arel	=  1 - arel

; get input from global accumulators

aL	rnd31 girscl, 0, ichn * 2		; add low level noise
	vincr aL, gaxL				; to avoid denormals
	clear gaxL

; read and convert controllers

$ZCTRL(kvol'ichn'7)			; volume 1
$ZCTRL(kvol2'ichn'24)			; volume 2

kvol	=  kvol * kvol / 16129
kvol2	tablei kvol2, 1

; apply amplitude control and parametric equalizers (if enabled)

kvol	=  iamp * kvol * kvol2

aL	=  aL * kvol

	if (imode1 < -0.5) goto l01
$ZCTRL(keq1f'ichn'26)			; EQ 1 frequency
$ZCTRL(keq1l'ichn'27)			; EQ 1 level
$ZCTRL(keq1q'ichn'28)			; EQ 1 Q
keq1f	tablei keq1f, 2			; convert controller values
keq1l	tablei keq1l, 3
keq1q	tablei keq1q, 4
aL	pareq aL, keq1f, keq1l, keq1q, imode1
l01:
	if (imode2 < -0.5) goto l02
$ZCTRL(keq2f'ichn'29)			; EQ 2 frequency
$ZCTRL(keq2l'ichn'30)			; EQ 2 level
$ZCTRL(keq2q'ichn'31)			; EQ 2 Q
keq2f	tablei keq2f, 2			; convert controller values
keq2l	tablei keq2l, 3
keq2q	tablei keq2q, 4
aL	pareq aL, keq2f, keq2l, keq2q, imode2
l02:

	out aL * arel			; de-click and write output

	endin

